<?php

if ( !class_exists( 'WP_List_Table' ) ) {
	require_once( ABSPATH . 'wp-admin/includes/class-wp-list-table.php' );
}

class XO_Login_Log_List_Table extends WP_List_Table {
	function __construct() {
		parent::__construct( array(
			'singular' => 'loginlog',
			'plural' => 'loginlogs',
			'ajax' => false
		) );
		wp_get_current_user();
	}

	function column_default( $item, $column_name ) {
		return $item[$column_name];
	}

	function column_success( $item ) {
		return $item['success'] ? __( 'Success', 'xo-security' ) : __( 'Failure', 'xo-security' );
	}

	function column_cb( $item ) {
		return sprintf( '<input type="checkbox" name="%1$s[]" value="%2$s" />', $this->_args['singular'], $item['id'] );
	}

	function get_columns() {
		global $current_user;

		$columns = array(
			'cb' => '<input type="checkbox" />',
			'login_time' => __( 'Date', 'xo-security' ),
			'success' => __( 'Result', 'xo-security' ),
			'ip_address' => __( 'IP address', 'xo-security' ),
			'lang' => __( 'Language', 'xo-security' ),
			'user_agent' => __( 'User Agent', 'xo-security' )
		);
		if ( $current_user->has_cap( 'administrator' ) ) {
			$columns['user_name'] = __( 'Username', 'xo-security' );
			$columns['failed_password'] = __( 'Password', 'xo-security' );
		}
		return $columns;
	}

	function get_sortable_columns() {
		$sortable_columns = array(
			'login_time' => array( 'logintime', true ),
			'success' => array( 'success', false ),
			'ip_address' => array( 'ipaddress', false ),
			'lang' => array( 'lang', false ),
			'user_name' => array( 'username', false ),
		);
		return $sortable_columns;
	}

	function get_bulk_actions() {
		$actions = array( 'delete' => __( 'Delete', 'xo-security' ) );
		return $actions;
	}

	// 結果フィルター ドロップ ダウンを表示します。
	function status_dropdown() {
		$status = isset( $_GET['status'] ) ? $_GET['status'] : '';
		echo '<label for="filter-by-status" class="screen-reader-text">' . __( 'Filter by results', 'xo-security' ) . '</label>';
		echo '<select name="status" id="filter-by-status">';
		echo '<option' . selected( $status, '', false ) . ' value="">' . __( 'All results', 'xo-security' ) . '</option>';
		printf( "<option %s value='%s'>%s</option>\n", selected( $status, '0', false ), '0', __( 'Failure', 'xo-security' ) );
		printf( "<option %s value='%s'>%s</option>\n", selected( $status, '1', false ), '1', __( 'Success', 'xo-security' ) );
		echo '</select>' . "\n";
	}

	// ユーザー フィルター ドロップ ダウンを表示します。
	function users_dropdown() {
		global $current_user;

		if ( $current_user->has_cap( 'administrator' ) ) {
			// 管理者
			$users = get_users();
			$user_login = isset( $_GET['u'] ) ? $_GET['u'] : '';
			echo '<label for="filter-by-user" class="screen-reader-text">' . __( 'Filter by user', 'xo-security' ). '</label>';
			echo '<select name="u" id="filter-by-user">';
			echo '<option' . selected( $user_login, '', false ) . ' value="">' . __( 'All users', 'xo-security' ) . '</option>';
			foreach ( $users as $user ) {
				printf( "<option %s value='%s'>%s</option>\n", selected( $user_login, $user->user_login, false ), esc_html( $user->user_login ), esc_html( $user->user_login ) );
			}
			echo '</select>' . "\n";
		} else {
			// 管理者以外
			echo '<label for="filter-by-user" class="screen-reader-text">' . __( 'Filter by user', 'xo-security' ). '</label>';
			echo '<select name="u" id="filter-by-user">';
			printf( '<option selected="selected" value="%s">%s</option>', esc_html( $current_user->user_login ), esc_html( $current_user->user_login ) );
			echo '</select>' . "\n";
		}
	}

	function extra_tablenav( $which ) {
		echo '<div class="alignleft actions">';
		if ( 'top' == $which && !is_singular() ) {
			$this->status_dropdown();
			$this->users_dropdown();
			submit_button( __( 'Filter', 'xo-security' ), 'button', 'filter_action', false, array( 'id' => 'post-query-submit' ) );
		}
		echo '</div>';
	}

	function process_bulk_action() {
		global $wpdb;

		$loginlog_table = $wpdb->base_prefix . XO_SECURITY_LOGINLOG_TABLE_NAME;
		if ( 'delete' === $this->current_action() ) {
			$ids = isset( $_GET['loginlog'] ) ? $_GET['loginlog'] : false;
			if ( !is_array( $ids ) ) {
				$ids = array( $ids );
			}
			$ids = implode( ',', $ids );
			if ( !empty( $ids ) ) {
				$wpdb->query( "DELETE FROM `{$loginlog_table}` WHERE id IN($ids);" );
			}
		}
	}

	function prepare_items() {
		global $wpdb;
		global $current_user;

		$loginlog_table = $wpdb->base_prefix . XO_SECURITY_LOGINLOG_TABLE_NAME;
		$per_page = $this->get_items_per_page( 'login_log_per_page', 100 );
		$status = isset( $_GET['status'] ) ? $_GET['status'] : '';

		if ( $current_user->has_cap( 'administrator' ) ) {
			$user_login = isset( $_GET['u'] ) ? $_GET['u'] : '';
		} else {
			$user_login = $current_user->user_login;
		}

		list( $columns, $hidden ) = $this->get_column_info();
		$sortable = $this->get_sortable_columns();
		$this->_column_headers = array( $columns, $hidden, $sortable );

		$this->process_bulk_action();

		// WHERE 句
		$status = esc_sql( $status );
		$user_login = esc_sql( $user_login );
		$where = ($status == '') ? '' : "success={$status}";
		$where .= ($user_login == '') ? '' : (($where == '') ? '' : " AND ") . "user_name='{$user_login}'";
		$where = ($where == '') ? '' : ' WHERE ' . $where;

		// 全データ件数を取得する
		$total_items = intval( $wpdb->get_var( "SELECT count(*) FROM `{$loginlog_table}`{$where}" ) );

		// 1ページ分のデータを取得する
		$orderby = (!empty( $_REQUEST['orderby'] )) ? $_REQUEST['orderby'] : 'logintime';
		$order = (!empty( $_REQUEST['order'] )) ? $_REQUEST['order'] : 'desc';
		$current_page = $this->get_pagenum();
		$start = ($current_page - 1) * $per_page;
		$rows = $wpdb->get_results( $wpdb->prepare(
			"SELECT `id`,`success`,`login_time` AS `logintime`,`ip_address` AS `ipaddress`,`lang`,`user_agent` AS `useragent`,`user_name` AS `username`,`failed_password` AS `failedpassword`" .
			"FROM {$loginlog_table}{$where} ORDER BY `{$orderby}` {$order} LIMIT %d, %d;", $start, $per_page
		), 'ARRAY_A' );
		foreach($rows as $row) {
			$item = array(
				'id' => $row['id'],
				'success' => $row['success'],
				'login_time' => $row['logintime'],
				'ip_address' => $row['ipaddress'],
				'lang' => $row['lang'],
				'user_agent' => $row['useragent'],
				'user_name' => $row['username'],
				'failed_password' => esc_html( $row['failedpassword'] )
			);
			$this->items[] = $item;
		}

		$this->set_pagination_args( array( 'total_items' => $total_items, 'per_page' => $per_page, 'total_pages' => ceil( $total_items / $per_page ) ) );
	}
}
